# InspectorGadget

[InspectorGadget]() is a tool designed to obtain an estimate of the performance and the memory cost of masking gadgets.

It comprises 5 folder/files:

- [`default_config_group_file.toml`](#default_config_group_file)
- [`examples_hardware_config`](#examples_hardware_config)
- [`examples_masking_config`](#examples_masking_config)
- [`gogogadgets`](#gogogadgets)
- [`inspector.py`](#inspector.py)


<h2 id="default_config_group_file"><code>default_config_group_file.toml</code></h2>

This is an example configuration file for all groups (currently integrated in InspectorGadget), it simply assign each 
group with the first matching gadget inside the `gogogadgets` library. This file can be automatically 
generated with the tool:

`python3 inspector.py --create_default_group_conf`

see **Package Requirements** section before running this command.


<h2 id="examples_hardware_config"><code>examples_hardware_config</code></h2>

This folder contains examples of microcontroller configurations, with 2 configuration files mimicking 
a Cortex M3 microcontroller and 1 configuration file related an simple abstract operation count model. It is 
possible to integrate other microcontroller model using one of these examples as a template.

<h2 id="examples_masking_config"><code>examples_masking_config</code></h2>

This folder contains examples of masking configurations. The "kyber_params" file contains the 
masking parameters for Kyber. As before, we can add any masking configuration we like to this tool, 
using one of these files as a template.

<h2 id="gogogadgets"><code>gogogadgets</code></h2>

This folder is the gadget library, it is made up of several elements necessary for adding and listing gadgets.

- `parameters.py`: Contains the class of masking parameters, which can change when a gadget's 
estimation is run
- `atomic_operations.py`: Defines the cost of each atomic operation, from a given microcontroller 
configuration file
- `gadget_dict.py`: The gadgets integrated into this tool are added to a gadget dictionary. This file 
contains all the functions required to manipulate this dictionary, such as adding a new gadget, 
listing all possible sub-gadget configurations for a given gadget, etc.
- `gadget_class.py`: (To complete)
- `gadgets`: This folder contains all the gadget definitions integrated into inspectorGadget
(a single file by gadget group). 

It is possible to integrate any gadget we wish, referring to the examples given in our paper and 
constructing the performance and memory cost functions of our gadget in such a way that their 
construction is consistent with that of the gadgets we wish to compare.

To add the gadget to the dictionary of estimable gadgets, use the following command:

```
gadgets_dict.add_new_gadget( \
	gadget_func, \
	input_type, \
	output_type, \
	name, \
	cost_estimation, \
	memory_estimation, \
	ref, \
	security_lvl = None \
	)
```
	
With:

```
	gadget_func (string)         : Functionality of the gadget
	input_type (sharing_type)    : Input type of the gadget
	output_type (sharing_type)   : Output type of the gadget
	name (string)                : Name of the gadget
	cost_estimation (function)   : #CPU cycles of the gadget
	memory_estimation (function) : memory cost of the gadget
	ref (string)                 : Link to the reference article
	security_lvl (security_level): Security level of the gadget
```

<h2 id="inspector.py"><code>inspector.py</code></h2>

The `inspector.py` file is used to launch estimates of gadgets or groups of gadgets 
present in the tool.

*** 

## Example of inspectorGadget use with `inspector.py`

### Package Requirements

inspectorGadget requires python version 3.8 or higher.

You will also need to install the tqdm and tomlkit packages. E.g. using pip:

```
% python3 -m pip install tomlkit tqdm
```

### Example

The `inspector.py` file takes a certain number of arguments to allows to estimate a gadget or a 
group of gadgets taking into account all the parameters seen previously. These parameters are all 
listed in the main function.

To give an example of use of our tool, here is the command that should be run to 
obtain the performance and the memory cost estimates of:

- gadget key:  KyberCompress256coeffsS2S1_CoronGMZ22_13 
- number of shares: 3
- modeled microcontroller: Cortex M3 (with rand generation cost 32 CCE)
- masking parameters: Kyber

```
% python3 inspector.py --component_config_file "examples_hardware_config/Cortex_M3_rand=32.toml" 
					 --params_config_file    "examples_masking_config/kyber_params.toml" 
					 --gadget_func           "KyberCompress256coeffs" 
					 --input_type            "Arithmetic" 
					 --output_type           "Boolean" 
					 --gadget_name           "CoronGMZ22_13" 
					 --list_n                "[3]"
```

Here, we do not specify which sub-gadgets must be used for the KyberCompress256coeffsS2S1_CoronGMZ22_13 estimation. 
The tool therefore returns the estimates made for all possible configurations of sub-gadgets. 

It is possible to impose the security level of a sub-gadget. In this case, if one of the possible configurations uses a sub-gadget that doesn't have the required security level, the memory and performance cost estimates returned are `None`.

If one wants to obtain results for a particular configuration, he needs to use the 
`--group_config_file argument` argument with a group configuration file like, e.g. `default_config_group_file.toml`,
which will force the tool to use the sub-gadgets assigned to the groups referenced in it.

It is possible to save the estimates obtained in a file by specifying its name and location.
If no gadget name is specified, the tool will estimate all gadgets belonging to the corresponding group.






