# Inspector Gadget
# Copyright (C) 2023  Camille Mutschler <camille.mutschler@lirmm.fr>
#                     Thomas Roche <thomas@ninjalab.io>
#                     Laurent Imbert <laurent.imbert@lirmm.fr>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from ..atomic_operations import *
from ..parameters import *
from .. import gadgets_dict
from ..gadgets_dict import sharing_type, security_level

def BsSecXORS1S1_Naive_cost(mask_params):
    """
    Estimation of the number of CPU cycles made by the bitsliced SecXOR 
    algorithm 

    Gives an estimate of the number of CPU cycles of the bitsliced SecXOR 
    algorithm, according to the parameters given at the input of the function.

    Reference code: https://github.com/uclcrypto/pqm4_masked/blob/master/crypto_kem/kyber768/m4fspeed_masked/gadgets.c
    Function: void masked_xor_c(size_t nshares, uint32_t *out, size_t out_stride,
                                const uint32_t *ina, size_t ina_stride, const uint32_t *inb,
                                size_t inb_stride)
    """
    
    n = mask_params.n
    xor_op = atomic_operations.get("xor_op").get("cost")
    # Loop computing the xor
    cost = n*xor_op
    return cost

def BsSecXORS1S1_Naive_memory(mask_params):
    """
    Estimation of the maximum of memory used in the bitsliced SecXOR 
    algorithm

    Gives an estimate of the maximum of memory used by the bitsliced SecXOR 
    algorithm, according to the parameters given at the input of the function. 

    Reference code: https://github.com/uclcrypto/pqm4_masked/blob/master/crypto_kem/kyber768/m4fspeed_masked/gadgets.c
    Function: void masked_xor_c(size_t nshares, uint32_t *out, size_t out_stride,
                                const uint32_t *ina, size_t ina_stride, const uint32_t *inb,
                                size_t inb_stride)
    """
    memory = 0
    return memory

gadgets_dict.add_new_gadget("BsSecXOR", sharing_type.Boolean, sharing_type.Boolean, "Naive", BsSecXORS1S1_Naive_cost, BsSecXORS1S1_Naive_memory, None, security_lvl = None)