# Inspector Gadget
# Copyright (C) 2023  Camille Mutschler <camille.mutschler@lirmm.fr>
#                     Thomas Roche <thomas@ninjalab.io>
#                     Laurent Imbert <laurent.imbert@lirmm.fr>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from ..atomic_operations import *
from ..parameters import *
from .. import gadgets_dict
from ..gadgets_dict import sharing_type, security_level

def SecAddS2S2_Naive_cost(mask_params):
    """
    Estimation of the number of CPU cycles made by the naive SecAdd algorithm 

    Gives an estimate of the number of CPU cycles of the naive SecAdd algorithm, 
    according to the parameters given at the input of the function.
    """
    
    n = mask_params.n
    add_op = atomic_operations.get("add_op").get("cost")
    modulo_Q_op = atomic_operations.get("modulo_Q_op").get("cost")
    # Loop computing a+b
    cost = n*add_op
    # Reduction modulo Q of all the shares
    cost += n*modulo_Q_op
    return cost

def SecAddS2S2_Naive_memory(mask_params):
    """
    Estimation of the maximum of memory used in the naive SecAdd algorithm

    Gives an estimate of the maximum of memory used by the naive SecAdd 
    algorithm, according to the parameters given at the input of the function. 
    """
    # there need to be a temp variable at least he size of the shares +1 to store the intermiate sum
    memory = mask_params.bitsize_shares + 1
    return memory

gadgets_dict.add_new_gadget("SecAdd", sharing_type.Arithmetic, sharing_type.Arithmetic, "Naive", SecAddS2S2_Naive_cost, SecAddS2S2_Naive_memory, None, security_lvl =  security_level.S_PI_NI)