# Inspector Gadget
# Copyright (C) 2023  Camille Mutschler <camille.mutschler@lirmm.fr>
#                     Thomas Roche <thomas@ninjalab.io>
#                     Laurent Imbert <laurent.imbert@lirmm.fr>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from ..atomic_operations import *
from ..parameters import *
from .. import gadgets_dict
from ..gadgets_dict import sharing_type, security_level

def SecMultConstModpS2S2_Naive_cost(mask_params):
    """
    Estimation of the number of CPU cycles made by the naive SecMultConstModp 
    algorithm

    Gives an estimate of the number of CPU cycles of the naive SecMultConstModp 
    algorithm, according to the parameters given at the input of the function.
    """
    
    n = mask_params.n
    mult_op = atomic_operations.get("mult_op").get("cost")
    modulo_Q_op = atomic_operations.get("modulo_Q_op").get("cost")
    # Loop computing const*share
    cost = n*mult_op
    # Reduction modulo Q of all the shares
    cost += n*modulo_Q_op
    return cost

def SecMultConstModpS2S2_Naive_memory(mask_params):
    """
    Estimation of the maximum of memory used in the naive SecMultConstModp 
    algorithm

    Gives an estimate of the maximum of memory used by the naive 
    SecMultConstModp algorithm, according to the parameters given at the input 
    of the function. 
    """
    # there need to be a temp variable at least twice the size of the shares to store the intermiate product
    memory = 2*mask_params.bitsize_shares
    return memory

gadgets_dict.add_new_gadget("SecMultConstModp", sharing_type.Arithmetic, sharing_type.Arithmetic, "Naive", SecMultConstModpS2S2_Naive_cost, SecMultConstModpS2S2_Naive_memory, None, security_lvl =  security_level.S_PI_NI)
