# Inspector Gadget
# Copyright (C) 2023  Camille Mutschler <camille.mutschler@lirmm.fr>
#                     Thomas Roche <thomas@ninjalab.io>
#                     Laurent Imbert <laurent.imbert@lirmm.fr>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from ..atomic_operations import *
from ..parameters import *
from .. import gadgets_dict
from ..gadgets_dict import sharing_type, security_level

def SecXORS1S1_Naive_cost(mask_params):
    """
    Estimation of the number of CPU cycles made by the SecXOR algorithm 

    Gives an estimate of the number of CPU cycles of the SecXOR algorithm, 
    according to the parameters given at the input of the function.
    """
    
    n = mask_params.n
    xor_op = atomic_operations.get("xor_op").get("cost")
    # Loop computing the xor
    cost = n*xor_op
    return cost

def SecXORS1S1_Naive_memory(mask_params):
    """
    Estimation of the maximum of memory used in the SecXOR algorithm

    Gives an estimate of the maximum of memory used by the SecXOR algorithm, 
    according to the parameters given at the input of the function. 
    """
    memory = 0
    return memory

gadgets_dict.add_new_gadget("SecXOR", sharing_type.Boolean, sharing_type.Boolean, "Naive", SecXORS1S1_Naive_cost, SecXORS1S1_Naive_memory, None, security_lvl =  security_level.S_PI_NI)
